/*
Prisoner's Dilemma (version 2.1)
written by Alexander M Kasprzyk, December '96
1996

This code was written for Toby Thain to demonstrate ways to accelerate
computation in the Iterated Prisoner's Dilemma.
*/

// include's
#include <Palettes.h>
#include <QdOffscreen.h>
#include "AppMacros.h"
#include "GridCopyPr.h"
#include "NumberStringUtilPr.h"

// defines
#define	_drawIterationString_	1		// turns on/off the drawing of the iteration string in the window
#define	_framesPerSecond_		1		// calculate the number of frames per second
#define	_traditionalColours_		1		// use the traditional colour scheme
#define	_seedCentre_			1		// 1= seed with one defector in the centre, 0 = use random seed (kInitialDefectors)
#define	kInitialDefectors		-32700	// the initial probability of defectors (probablilty between -32767 and +32767) - only used if _seedCentre_ = 0
#define	kHSize				200		// horizontal grid size
#define	kVSize				200		// vertical grid size
#define	kTotalSize				40000	// must be set to kVSize * kHSize
#define	kScale				2		// the drawing scale - set to 1 or 2
// reward matrix values
#define	kCooperateCooperate		100		// both cooperate - default = 100
#define	kCooperateDefect		0		// cooperate, but the other one defects - default = 0
#define	kDefectDefect			0		// both defectors - defualt = 0
#define	kDefectCooperate		185		//	try 115, 135, 155, 177, 185, 190, 201
									//	to see rapid change in behaviour, try 179, 180, 181
// function prototypes
void			main				( void );
static void	InitToolbox		( void );
static Boolean	InitMemory		( void );
static void	DisposeMemory		( void );
static void	InitGrid			( void );
static void	OneIteration		( void );

// global variables
#if _drawIterationString_
Str255		iterationStr = "\p1";	// iteration counter string
#endif
#if _framesPerSecond_
unsigned long	ticks;				// start ticks
Str255		ticksStr = "\p";		// ticks string
unsigned long	iterations = 0;			// number of iterations
unsigned long	totalSecs = 0;			// total number of seconds we've run for (approx)
#endif
GWorldPtr		theGWorld;			// the GWorld for direct drawing to
WindowRef	theWindow;			// the window to display the GWorld in (may need to replace "WindowRef" with "WindowPtr" on some compilers)
RBlockDataPtr	gridMove;				// grid move data
unsigned char	*oldGrid,				// the old grid array
			*newGrid,			// the new grid array
			colours[2][2],			// colour values
			**rowOffset;			// the row offset table for GWorld drawing
unsigned long	*neighbourArray;		// the cell neighbour array
short		defectorAwards[2] = {kDefectCooperate, kDefectDefect},			// defector payoff matrix
			cooperatorAwards[2] = {kCooperateCooperate, kCooperateDefect},	// cooperator payoff matrix
			*payoff;				// the payoff array

// main -	the program entry point
void main( void )
{
	Boolean	quitting = false;
	
	// initialize the application
	InitToolbox();
	if( !InitMemory() )
	{
		SysBeep(1);	// not enough memory
		ExitToShell();
	}
	InitGrid();
#if _framesPerSecond_
	ticks = TickCount() + 60;
#endif
	
	// loop in this little event loop
	while( !quitting )
	{
		EventRecord	theEvent;
		
		if( WaitNextEvent( everyEvent, &theEvent, 0, 0L ) )
		{
			WindowRef 		theWindow;
			short  			where;

			switch( theEvent.what )
			{
				case mouseDown:
					where = FindWindow( theEvent.where, &theWindow );
					switch( where )
					{
						case inGoAway:
							quitting = TrackGoAway( theWindow, theEvent.where );
							break;
						case inDrag:
							DragWindow( theWindow, theEvent.where, &qd.screenBits.bounds );
							break;
					}
					break;
				case updateEvt:
					theWindow = theWindow = (WindowRef)theEvent.message;
					BeginUpdate( theWindow );
					CopyBits( &((GrafPtr)theGWorld)->portBits, &theWindow->portBits, &theGWorld->portRect, &theGWorld->portRect, srcCopy, 0 );
					EndUpdate( theWindow );
					break;
			}
		}
		OneIteration();		// <- this is why we are here
	}
	
	// close down all the memory
	DisposeMemory();
}

// InitToolbox -	initializes the Mac toolboxes
static void InitToolbox( void )
{
	InitGraf( &qd.thePort );
	InitFonts();
	InitWindows();	
	InitMenus();
	InitCursor();
	TEInit();
	InitDialogs( 0L );
	GetDateTime( (unsigned long*)&qd.randSeed );
}

// InitMemory -	allocates the memory needed for the simulation
static Boolean InitMemory( void )
{
	RGBColor		col;
	Rect			theRect;
	
/* The following is a little check of the kTotalSize constant. It is not really needed, but
if you make a mistake setting kTotalSize it could prevent you from crashing the Mac.
It is possible to replace the size constants with variables, but it will effect the speed of the
application. */
if( (long)kHSize * (long)kVSize != kTotalSize )
	return( false );
	
	// create the new GWorld
	MSetRect( theRect, 0, 0, kHSize * kScale, kVSize * kScale );
	if( NewGWorld( &theGWorld, 8, &theRect, nil, nil, 0L ) != noErr )
		return( false );
	
	// lock down the GWorld and create drawing offset table
	LockPixels( MGetGWorldPixMap( theGWorld ) );
	if( !(rowOffset = (unsigned char **)NewPtr( sizeof( unsigned char * ) * (theRect.bottom + 1) )) )
		return( false );
	else
	{
		register unsigned long	count;
		register Ptr			baseAddr;
		register short			rowBytes;
		
		baseAddr = GetPixBaseAddr( MGetGWorldPixMap( theGWorld ) );
		rowBytes = MGetRowBytes( theGWorld );
		
		for( count = 0; count <= theRect.bottom; count++ )
			*(rowOffset + count) = (unsigned char *)(baseAddr + rowBytes * count);
	}
	
	// create new window
	MOffsetRect( theRect, 40, 40 );
	if( !(theWindow = NewCWindow( 0L, &theRect, "\pPrisoner's Dilemma", true, documentProc, (WindowRef)-1L, true, 0L )) )
		return( false );
	
	// create the block move data
	if( !(gridMove = (RBlockDataPtr)NewPtr( sizeof( RBlockData ) )) )
		return( false );
	
	//  create the grid arrays
	if( !(oldGrid = (unsigned char *)NewPtr( sizeof( unsigned char ) * kTotalSize )) )
		return( false );
	if( !(newGrid = (unsigned char *)NewPtr( sizeof( unsigned char ) * kTotalSize )) )
		return( false );
	SetRBlockData( gridMove, GetPtrSize( (Ptr)newGrid ) );
	
	// create the payoff array
	if( !(payoff = (short *)NewPtr( sizeof( short ) * kTotalSize )) )
		return( false );
		
	// create the neighbour array
	if( !(neighbourArray = (unsigned long *)NewPtr( sizeof( unsigned long ) * kTotalSize * 8 )) )
		return( false );
	else
	{
		long			countx, county;
		unsigned long	*array = neighbourArray;
		
		for( county = 0; county < kVSize; county++ )
			for( countx = 0; countx < kHSize; countx++ )
			{
				long		offx, offy;
				
				for( offy = -1; offy <= 1; offy++ )
					for( offx = -1; offx <= 1; offx++ )
						if( !(offx == 0 && offy == 0) )
						{
							long		realx = offx + countx, realy = offy + county;
						
							if( realx < 0 )
								realx = kHSize - 1;
							else if( realx >= kHSize )
								realx = 0;
							if( realy < 0 )
								realy = kVSize - 1;
							else if( realy >= kVSize )
								realy = 0;
					
							*array++ = realy * kHSize + realx;
						}
			}
	}
	
	// copy the window's colour table ID into the GWorld's ID to speed up CopyBits
	SetGWorld( (CGrafPtr)theWindow, nil );
	MCTSeedSmash( theGWorld );		// !!! may cause a crash on some monitors !!!
	SetGWorld( theGWorld, nil );
	
	// set the GWorld font and size
	TextFont( geneva );
	TextSize( 10 );
	
	// get the direct to screen colour values
#if _traditionalColours_
	MGetBlueColor( col );				// is a cooperator; was a cooperator
	colours[0][0] = Color2Index( &col );
	MGetRedColor( col );					// is a defector; was a defector
	colours[1][1] = Color2Index( &col );
	MGetGreenColor( col );				// is a cooperator; was a defector
	colours[0][1] = Color2Index( &col );
	MGetYellowColor( col );				// is a defector; was a cooperator
	colours[1][0] = Color2Index( &col );
#else
	MSetRGBColor( col, 0, 0xFFFF, 0 );	// is a cooperator; was a cooperator
	colours[0][0] = Color2Index( &col );
	MSetRGBColor( col, 0, 0, 0xFFFF );	// is a defector; was a defector
	colours[1][1] = Color2Index( &col );
	MSetRGBColor( col, 0, 0, 0x7777 );	// is a cooperator; was a defector
	colours[0][1] = Color2Index( &col );
	MSetRGBColor( col, 0, 0x7777, 0 );	// is a defector; was a cooperator
	colours[1][0] = Color2Index( &col );
#endif
	
	return( true );
}

// DisposeMemory -	call to release all the memory
static void DisposeMemory( void )
{
	UnlockPixels( MGetGWorldPixMap( theGWorld ) );
	DisposeGWorld( theGWorld );
	DisposeWindow( theWindow );
	DisposePtr( (Ptr)oldGrid );
	DisposePtr( (Ptr)newGrid );
	DisposePtr( (Ptr)rowOffset );
	DisposePtr( (Ptr)neighbourArray );
	DisposePtr( (Ptr)payoff );
}

// InitGrid -	randomizes new grid
static void InitGrid( void )
{
#if _seedCentre_
	*(newGrid + kHSize * (kVSize / 2) + kHSize / 2) = 1;
#else
	unsigned char	*newg = newGrid;
	long			count;
	
	for( count = 0; count < kTotalSize; count++ )
		*newg++ = (Random() < kInitialDefectors);
#endif
}

// OneIteration -	call to advance the CA by one iteration
static void OneIteration( void )
{
	register long			count;
	register unsigned char	*oldg = oldGrid, *newg = newGrid;
	register unsigned long	*neighbour = neighbourArray;
	short				*pay = payoff;
	short				*mode;
	unsigned char			**row = rowOffset;
	register short			myPay;
	
	// copy the new grid over the old grid
	RBlockMove( (Ptr)newg, (Ptr)oldg, gridMove );
	
	// calculate payoffs for each player on the board
	for( count = 0; count < kTotalSize; count++, pay++ )
	{
		if( *oldg )
			mode = defectorAwards;
		else
			mode = cooperatorAwards;
		*pay = *(mode + *oldg++);
		*pay += *(mode + *(oldGrid + *neighbour++));
		*pay += *(mode + *(oldGrid + *neighbour++));
		*pay += *(mode + *(oldGrid + *neighbour++));
		*pay += *(mode + *(oldGrid + *neighbour++));
		*pay += *(mode + *(oldGrid + *neighbour++));
		*pay += *(mode + *(oldGrid + *neighbour++));
		*pay += *(mode + *(oldGrid + *neighbour++));
		*pay += *(mode + *(oldGrid + *neighbour++));
	}
	
	neighbour = neighbourArray;
	pay = payoff;
	oldg = oldGrid;
		
	// find largest payoff in each area and calculate new strategies
	// draw player's strategies/colour to the GWorld and copy the GWorld to the window
	for( count = 0; count < kVSize; count++, row += kScale )
	{
		register long		counth;
		
		for( counth = 0; counth < kHSize * kScale; counth += kScale )
		{
#if kScale == 2
			register unsigned char	color;
#endif
			myPay = *pay++;
			if( myPay < *(payoff + *neighbour) )
			{
				*newg = *(oldGrid + *neighbour);
				myPay = *(payoff + *neighbour);
			}
			if( myPay < *(payoff + *(neighbour + 1)) )
			{
				*newg = *(oldGrid + *(neighbour + 1));
				myPay = *(payoff + *(neighbour + 1));
			}
			if( myPay < *(payoff + *(neighbour + 2)) )
			{
				*newg = *(oldGrid + *(neighbour + 2));
				myPay = *(payoff + *(neighbour + 2));
			}
			if( myPay < *(payoff + *(neighbour + 3)) )
			{
				*newg = *(oldGrid + *(neighbour + 3));
				myPay = *(payoff + *(neighbour + 3));
			}
			if( myPay < *(payoff + *(neighbour + 4)) )
			{
				*newg = *(oldGrid + *(neighbour + 4));
				myPay = *(payoff + *(neighbour + 4));
			}
			if( myPay < *(payoff + *(neighbour + 5)) )
			{
				*newg = *(oldGrid + *(neighbour + 5));
				myPay = *(payoff + *(neighbour + 5));
			}
			if( myPay < *(payoff + *(neighbour + 6)) )
			{
				*newg = *(oldGrid + *(neighbour + 6));
				myPay = *(payoff + *(neighbour + 6));
			}
			if( myPay < *(payoff + *(neighbour + 7)) )
				*newg = *(oldGrid + *(neighbour + 7));
#if kScale == 2			
			color = colours[*newg++][*oldg++];
			
			*(*row + counth) = color;
			*(*row + counth + 1) = color;
			*(*(row + 1) + counth) = color;
			*(*(row + 1) + counth + 1) = color;
#else
#if kScale == 1
			*(*row + counth) = colours[*newg++][*oldg++];
#else
			ExitToShell();
#endif
#endif
			neighbour += 8;
		}
	}
	
#if _framesPerSecond_
	iterations++;
	if( TickCount() >= ticks )
	{
		unsigned long		reIt;
		Str255			deciStr;
		unsigned long		*lstrPos;
		unsigned char		*strPos;
		
		totalSecs++;
		reIt = iterations/totalSecs;
		NumToStringUChar( reIt, ticksStr );
		reIt = ((iterations - (reIt * totalSecs)) * 100) / totalSecs;
		NumToStringUChar( reIt, deciStr );
		if( deciStr[0] == 1 )
		{
			deciStr[2] = deciStr[1];
			deciStr[1] = '0';
		}
		strPos = &ticksStr[ticksStr[0] + 1];
		*strPos = '.';
		*(strPos + 1) = deciStr[1];
		*(strPos + 2) = deciStr[2];
		lstrPos = (unsigned long *)(&ticksStr[ticksStr[0] + 4]);
		*lstrPos = 0x20667073;	// hex for ' fps'
		ticksStr[0] += 7;
		ticks = TickCount() + 60;
	}
	MoveTo( 3, 10 );
	DrawString( ticksStr );
	ForeColor( whiteColor );
	MoveTo( 4, 11 );
	DrawString( ticksStr );
	ForeColor( blackColor );
#endif
#if _drawIterationString_
	// draw the iteration string
	myPay = (kHSize * kScale - StringWidth( iterationStr )) / 2;
	MoveTo( myPay, 10 );
	DrawString( iterationStr );
	ForeColor( whiteColor );
	MoveTo( myPay + 1, 11 );
	DrawString( iterationStr );
	ForeColor( blackColor );
	Add1ToString( iterationStr );
#endif
	
	// copy the GWorld to the window
	CopyBits( &((GrafPtr)theGWorld)->portBits, &theWindow->portBits, &theGWorld->portRect, &theGWorld->portRect, srcCopy, 0 );
}